%**************************************************************************************************
% Author: Md Asafuddoula :: http://seit.unsw.adfa.edu.au/research/sites/mdo/index.htm
% An Adaptive reference vector guided DBEA (ADBEA)
% Copyright 2016: UNSW,Canberra.
% Last Updated: 23/02/2016 @Md Asafuddoula
% Highlights of this revised form:
% (a) Each weight direction is tagged with random offspring
% solutions. Preferred weight directions are identified for the offspring
% solutions based on minimum perpendicular distance.
% Weight direction having no solution after a given number of iterations or dominated is considered
% empty and it is being moved to a maximum angle solution of its neighbor among its tagged offpring.
% (b) Nadir identified using ND
% (C) Minimization considered in this formulation
% (d) Please make sure objectives are positive throughout the search space
% (e) Parent selection: P1 ordered and P2 random
%**************************************************************************************************

function fitness=gDBEA(def)
tic;
rand('twister', def.seed);
% Get the problem definition
prob=load_problem_definition(def);
def.nf=prob.nf;
% 2-layer weight direction generation
[def.popSize,w]=direction_vector(def.p1, def.p2, def.nf);
% [distMat]=computeDistanceTable(w);
for i=1:size(w,1)
    w(i,:)=w(i,:)/norm(w(i,:));
end
[thetaTable]=computeThetaTable(w);
if(prob.nf>3), def.display=0; end
def.maxFunCount=def.gen*def.popSize;
nChild=2;
def.gen=round((def.maxFunCount-def.popSize)/(nChild*def.popSize));
% Initialize the population of solutions
funCount=0;
[x,fitness]=initialize_pop(prob,def);
funCount=funCount+size(fitness,1);
% Initialize the ideal and nadir point
idealpoint=ones(1,prob.nf)*1e10; nadirpoint=ones(1,prob.nf)*1e-10;
[idealpoint,nadirpoint]=init_idealpoint(fitness,prob,idealpoint,nadirpoint);
% statistics of number of solution associate to each reference direction
gen=1; nc=zeros(def.gen,size(w,1));
% statistics of reference direction domination
dom_count=zeros(def.gen,size(w,1));
% frequency of reference direction adaptation
w_unused=ones(size(w))*(-1);
% global variale to store the results
global alldata;
if(def.statistics_all)
    alldata=zeros(def.gen*def.popSize,1+1+prob.nx+prob.ng+1+prob.nf);
else
    alldata=zeros(1*def.popSize,1+1+prob.nx+prob.ng+1+prob.nf);
end
show_progress(fitness,prob,def,idealpoint,nadirpoint,w,1,gen,(def.displayReferenceDirection+1),funCount);
while(funCount<def.maxFunCount)
    
    s=randperm(def.popSize);
    nr=ones(1,def.popSize*nChild)*-1; d2=zeros(1,def.popSize*nChild);
    count=1; child_fitness=zeros(def.popSize*nChild,prob.nf+prob.ng+1);
    child_x=zeros(def.popSize,prob.nx);
    for i=1:size(w,1)
        p1=s(i);
        p2=roulettewheelselection(thetaTable(p1,:));
        p3=roulettewheelselection(thetaTable(p2,:));
        %DE operator for child generation
        if(rand<def.operator_pr)
        [child_x(count,:)]= DE_operator(x,p1,p2,p3,def.CR,def.F,prob.range);
        
        [child_fitness(count,:)]=Evaluate(child_x(count,:),prob,def);
        [idealpoint,nadirpoint]=update_ideal_nadir_point(fitness, child_fitness(count,:), idealpoint, nadirpoint, prob, gen, def);
        [x,fitness, nr(count),d2(count+1), w, w_unused]=updatePopulation(x, fitness, child_x(count,:), child_fitness(count,:), w, w_unused, idealpoint, nadirpoint, prob, funCount,def.maxFunCount);
        count=count+1;
        funCount=funCount+1;
        % generate the child solutions via SBX
        else
        [child_x(count,:),child_x(count+1,:)]=SBX_crossover(x(p1,:), x(p2,:), prob,def);
%       only one child goes through mutation
        [child_x(count,:)]=polymutation(child_x(count,:),prob,def);
        [child_fitness(count,:)]=Evaluate(child_x(count,:),prob,def);
        [child_fitness(count+1,:)]=Evaluate(child_x(count+1,:),prob,def);
        
        % update the ideal and extreme points
        [idealpoint,nadirpoint]=update_ideal_nadir_point(fitness, child_fitness(count,:), idealpoint, nadirpoint, prob, gen, def);
        [idealpoint,nadirpoint]=update_ideal_nadir_point(fitness, child_fitness(count+1,:), idealpoint, nadirpoint, prob, gen, def);
        % update the population
        [x,fitness, nr(count), d2(count), w, w_unused]=updatePopulation(x, fitness, child_x(count,:), child_fitness(count,:), w, w_unused, idealpoint, nadirpoint, prob, funCount,def.maxFunCount);
        [x,fitness, nr(count+1),d2(count+1), w, w_unused]=updatePopulation(x, fitness, child_x(count+1,:), child_fitness(count+1,:), w, w_unused, idealpoint, nadirpoint, prob, funCount,def.maxFunCount);
        funCount=funCount+2;
        count=count+2;
        end
    end
    [nc, dom_count]=associatedSolutionsCount(nr, fitness, nc, dom_count, prob, gen);
    drawnow;
%     % Adaptive weight update procedure
    if(mod(gen,def.gen_fr)==0)
        [w, w_unused]=adaptive_ref_direction_update(w, w_unused, nr, d2,x,fitness,child_x, child_fitness, idealpoint, nadirpoint, nc, dom_count, def.gen_fr, gen, prob);
    end
%     % display the progress of search
    show_progress(fitness,prob,def,idealpoint,nadirpoint,w,2,gen,(def.displayReferenceDirection+1),funCount);
    % pointgen(gen,:) = [gen idealpoint nadirpoint];
    
    % Writing the population information to a file
    write_progress(x, fitness, prob, def, funCount, gen, def.problem_name);
    
    clc;
    disp(strcat(num2str(round((funCount/def.maxFunCount)*100)),'%'));
%     disp(num2str(IGD(fitness(:,1:prob.nf),w,def.problem_name)));
    gen=gen+1;
end

% clc;
%  disp(num2str(IGD(fitness(:,1:prob.nf),w)));

timeexec=toc;
dlmwrite('runtime.dat',timeexec,'precision','%10.4f','delimiter',' ');
feas_id= fitness(:,prob.nf+prob.ng+1)==0;
ffeas = fitness(feas_id,1:prob.nf);
% Eliminating all solutions worse off than Nadir
tmp=sum(ffeas-repmat(nadirpoint,size(ffeas,1),1)>=0,2);
idx=find(tmp==prob.nf);
[~,idx1] = nd_rank_one(ffeas);
idx1=setdiff(idx1,idx);
ndffeas = ffeas(idx1,:);
% pointgen(gen,:) = [gen min(ndffeas) max(ndffeas)];
% dlmwrite('idealnadir.dat',pointgen,'precision','%10.4f','delimiter',' ');
dlmwrite('ndfeasibleobj.dat',ndffeas,'precision','%10.4f','delimiter',' ');
save_figure(def.display,strcat(def.problem_name,'_final_tagged_solutions'));
show_progress(fitness,prob,def,idealpoint,nadirpoint,w,1,gen,(def.displayReferenceDirection+1),funCount);
save_figure(def.display,strcat(def.problem_name,'_Pareto_front'));
return

function save_figure(flag,name)
if(flag==1)
    set(gca,'FontSize',16,'FontName','Arial');
    saveas(gca, strcat(name,'.fig'));
    print('-depsc','-r150',strcat(name,'.eps'));
end
close;
return


function v=IGD(P,PS,p_name)
if(strcmp(p_name,'dtlz1')||strcmp(p_name,'c1dtlz1') )
    ip=zeros(1,size(P,2));
    np=ones(1,size(P,2))*0.5;
    P=(P-repmat(ip,size(P,1),1))./(repmat(np,size(P,1),1)-repmat(ip,size(P,1),1));
    for i=1:size(PS,1)
        PS(i,:)=PS(i,:)/sum(PS(i,:));
    end
end
size_PS=size(PS,1);
for j=1:size(PS,1)
    for k=1:size(P,1)
        D(k)=sqrt(sum((PS(j,:)-P(k,:)).^2));
    end
    val(j)=min(D);
end
v=sum(val)/size_PS;
return


function [nc,dom_count]=associatedSolutionsCount(nr, fitness, nc, dom_count, prob, gen)


%non-domination count for each reference direction
try
    % use faster non-dominated rank one solutions
    id=nd_rank_one_MEX(fitness(:,1:prob.nf)');
    dom_count(gen,:)=id~=-1;
catch
    [~,nd_id]=nd_rank_one(fitness(:,1:prob.nf));
    dom_count(gen,nd_id)=1;
end

%identify the niche count
for i=1:size(fitness,1)
    a=find(nr==i);
    nc(gen,i)=length(a);
    if(fitness(i,prob.nf+prob.ng+1)~=0)
        dom_count(gen,i)=0;
    end
end

return

%Adaptive weight direction update procedure
%deletion and addition of weight vector
function [w, w_unused]=adaptive_ref_direction_update(w, w_unused, nr, d2, x, fitness,child_x, child_fitness, idealpoint, nadirpoint, nc, dom_count, gen_fr, gen,prob)

avg_nc=mean(nc((gen-gen_fr+1):gen,:));
avg_dom=mean(dom_count((gen-gen_fr+1):gen,:));

% identify the location of the reference direction move
new_loc=ones(1,size(w,1))*-1;
for i=1:size(w,1)
    a=find(nr==i);
    if(length(a)>1)
        [~,max_id]=max(d2(a));
        new_loc(i)=a(max_id);
    end
end

% deletion of reference direction and addition of new direction
for i=1:size(w,1)
    % reference direction with nich count zero or dominated for a preiod of generations
    if(avg_nc(i)==0 || avg_dom(i)==0) % i-th reference direction
        % add this refernce direction to unused reference set
        if(w_unused(i,1)==-1)
            w_unused(i,:)=w(i,:);
        end
        % move the reference direction to its neighbor if it's crowded
        d=zeros(1,size(w,1));
        for j=1:size(w,1)
            d(j)=norm(w(i,:)-w(j,:));
        end
        [~,I]=sort(d);
        %find the closest crowded neighbor
        for j=2:length(I)
            if(new_loc(I(j))~=-1)
                % generate weight vector close to the closet neighbor's dinstance/2  and break
                % the loop
                new_w=(child_fitness(new_loc(I(j)),1:prob.nf)-idealpoint)./(nadirpoint-idealpoint);
                new_w=new_w/norm(new_w);
                %if the weight direction doesn't exist
                if(sum(sqrt(sum((w-repmat(new_w,size(w,1),1)).^2,2))==0)==0)
                    w(i,:)=new_w;
                    %assign the solution to relocated reference vectors
                    x(i,:)=child_x(new_loc(I(j)),:);
                    fitness(i,:)=child_fitness(new_loc(I(j)),:);
                    new_loc(I(j))=-1; % this location has been used
                    break;
                end
            end
        end
    end
end


% figure(3);
% plot(1:size(w,1),avg_nc,'r*-');
% xlabel('Solution no');
% ylabel('Average niche count');
%
% figure(4);
% plot(1:size(w,1),avg_dom,'k*-');
% xlabel('Solution no');
% ylabel('Average non-domination count');
% hold off;

return


% Compute the theta value from a reference direction to others and reverse
% the value to reflect the closet solution is preferred with higher
% probability
function [thetaTable]=computeThetaTable(w)
thetaTable=zeros(size(w,1),size(w,1));
for i=1:size(w,1)
    maxSoFar=-inf;
    for j=1:size(w,1)
        if(i~=j)
            thetaTable(i,j)=subspace(w(i,:)',w(j,:)');
        else
            thetaTable(i,j)=0;
        end
        if(thetaTable(i,j)>maxSoFar)
            maxSoFar=thetaTable(i,j);
        end
    end
    for j=1:size(w,1)
        thetaTable(i,j)=maxSoFar-thetaTable(i,j);
    end
    
end

return

function [distMat]=computeDistanceTable(w)
distMat=zeros(size(w,1),size(w,1));
dist=zeros(1,size(w,1));
for i=1:size(w,1)
    for j=1:size(w,1)
        if(i~=j)
            dist(j)=norm(w(i,:)-w(j,:));
        else
            dist(j)=0;
        end
    end
    [~,I]=sort(dist);
    distMat(i,:)=I;
end

return

%Associate solutions to reference directions based on theta threshold, non-dominance and d2 distance Matlab version
function [id, d2, d1, w, w_unused]=associateCandidateSolution(child_fitness, fitness, w, w_unused, idealpoint, nadirpoint,prob)

N1=size(child_fitness,1);
F1=(child_fitness(:,1:prob.nf)-ones(N1,1)*idealpoint)./(ones(N1,1)*nadirpoint-ones(N1,1)*idealpoint);

[d1]=distanceD1(F1,w);
[d2]=distanceD2(F1,d1);
id=zeros(1,size(F1,1));
for i=1:size(F1,1)
    d2_unused=inf(1,size(w,1));
    d1_unused=inf(1,size(w,1));
    for k=1:size(w,1)
        if(w_unused(k,1)~=-1)
            d1_unused(k)=distanceD1(F1(i,:),w_unused(k,:));
            d2_unused(k)=distanceD2(F1(i,:),d1_unused(k));
        end
    end
    [~,id(i)]=min(d2(i,:));
    [~,id2]=min(d2_unused);
    if(child_fitness(:,prob.nf+prob.ng+1)==0)
        if(d2_unused(id2)<d2(i,id(i)) && nd_check(child_fitness(:,1:prob.nf),fitness(:,1:prob.nf)))
            w(id2,:)=w_unused(id2,:);
            w_unused(id2,:)=-1;
            d2(i,id(i))=d2_unused(id2);
            d1(i,id(i))=d1_unused(id2);
            id(i)=id2;
        end
    end
end
return

function [flag]=nd_check(cf,f)
flag=1;
m=size(f,2);
for i=1:size(f,1)
    if(sum(cf>f(i,:))==m)
        flag=0;
        break;
    end
end
return

function [flag]=nd_check_self(f,id)
flag=1;
m=size(f,2);
for i=1:size(f,1)
    if(i==id), continue;
    elseif(sum(f(id,:)>f(i,:))==m)
        flag=0;
        break;
    end
end
return

function [idealpoint,nadirpoint]=init_idealpoint(fitness,prob,idealpoint,nadirpoint)
f=fitness(fitness(:,prob.nf+prob.ng+1)==0,1:prob.nf);
if(~isempty(f))
    f=nd_rank_one(f);
    if(size(f,1)>1)
        idealpoint=min(f);
        nadirpoint=max(f);
    else
        idealpoint=f;
        nadirpoint=f;
    end
end
return

% Update the idealpoint and the maximum point
function [idealpoint,nadirpoint]=update_ideal_nadir_point(fitness,child_fitness,idealpoint,nadirpoint,prob,gen,def)
Beta=1/def.gen_fr;  %frequency of nadir update with previous nadir
f=[fitness(fitness(:,prob.nf+prob.ng+1)==0,1:prob.nf); child_fitness(child_fitness(:,prob.nf+prob.ng+1)==0,1:prob.nf)];
if ~isempty(f)
    idealpoint=min([f;idealpoint]);
    if(size(f,1)==1)
        nd_max=f;
    else
        try
            % faster nd rank one computation
            id=nd_rank_one_MEX(f');
            index= id~=-1;
            f_nd=f(index,:);
            nd_max=max(f_nd);
        catch
            % slower nd rank one computation
            f_nd=nd_rank_one(f);
            nd_max=max(f_nd);
        end
    end
    % with a freuqency of 0.1, nadirpoint is check whether it minimizes
    % from its previous nadir for stability
    if mod(gen,def.gen*Beta)
        nadirpoint=min(nd_max,nadirpoint);
    else
        nadirpoint=nd_max;
    end
end

return

% Compare the solutions
function [b]=compare_objective(d1_child,d2_child,d1_parent,d2_parent,eps)
b=false;
if(d2_child==d2_parent || d2_child<eps && d2_parent<eps)
    if(d1_child<d1_parent)
        %Child winner
        b=true;
    end
elseif (d2_child<d2_parent)
    %Child winner
    b=true;
end
return

% Computing the threshold for constrained problems
function [e]=constraint_approach(fitness,prob)
popsize=size(fitness,1);
cv_max=sum(fitness(:,prob.ng+prob.nf+1));
cv_max=cv_max/popsize;
feasibility=(sum(fitness(:,prob.ng+prob.nf+1)==0))/popsize;
e=feasibility*cv_max;
return


% Updating the population based on a child pop and a pop.
function [x, fitness, j, min_d2, w, w_unused]=updatePopulation(x, fitness, child_x, child_fitness, w, w_unused,idealpoint,nadirpoint,prob,funCount, maxFunCount)

[j, d2, d1, w, w_unused]=associateCandidateSolution(child_fitness, fitness, w, w_unused, idealpoint, nadirpoint, prob);
% min distance obtain for the child solution among all referecne direction
min_d2=d2(j);
C=sum(w(j,:)==0)==(prob.nf-1);

theta=5;

if ((child_fitness(prob.nf+prob.ng+1)==0 && fitness(j,prob.nf+prob.ng+1)==0)...
        || (child_fitness(prob.nf+prob.ng+1)<constraint_approach(fitness,prob) &&...
        fitness(j,prob.nf+prob.ng+1)<constraint_approach(fitness,prob)))
    
    if(sum(child_fitness(1:prob.nf)<fitness(j,1:prob.nf),2)==prob.nf)
        x(j,:)=child_x;
        fitness(j,:)=child_fitness;
    elseif(nd_check(child_fitness(1:prob.nf),fitness(:,1:prob.nf))>nd_check_self(fitness(:,1:prob.nf),j))
        x(j,:)=child_x;
        fitness(j,:)=child_fitness;
    elseif(sum(child_fitness(1:prob.nf)<fitness(j,1:prob.nf),2)>=1)
        pf=(fitness(j,1:prob.nf)-idealpoint)./(nadirpoint-idealpoint);
        cf=(child_fitness(1:prob.nf)-idealpoint)./(nadirpoint-idealpoint);
        d1_parent=distanceD1(pf,w(j,:));
        d2_parent=distanceD2(pf,d1_parent);
        
        d1_child=d1(j);
        d2_child=d2(j);
%       eps=0;
%       [b]=compare_objective(d1_child,d2_child,d1_parent,d2_parent,eps);
        
        %% Apply ASF for the corner solution
        a=w(j,:);
        a(a==0)=1e-6;
        tch_parent=min(max(abs(ones(size(pf,1),1)*idealpoint-pf)./(ones(size(pf,1),1)*a),[],2));
        tch_child=min(max(abs(ones(size(cf,1),1)*idealpoint-cf)./(ones(size(cf,1),1)*a),[],2));
        %-------------------------------------------------------------------------------
        if(C && (tch_child<tch_parent))
            x(j,:)=child_x;
            fitness(j,:)=child_fitness;
        elseif(d1_child+theta*d2_child)<(d1_parent+theta*d2_parent)
            x(j,:)=child_x;
            fitness(j,:)=child_fitness;

        end
    end
    % for both infeasible solutions, solution with less infeasibility
    % will be preferred
else
    if(child_fitness(prob.nf+prob.ng+1)<fitness(j,prob.nf+prob.ng+1))
        x(j,:)=child_x;
        fitness(j,:)=child_fitness;
    else
        [~,I]=sort(fitness(:,prob.nf+prob.ng+1),'descend');
        for l=1:size(w,1)
            if(child_fitness(prob.nf+prob.ng+1) < fitness(I(l),prob.nf+prob.ng+1))
                x(I(l),:)=child_x;
                fitness(I(l),:)=child_fitness;
                break;
            end
        end
    end
end


return


function [d1]=distanceD1(f,w)
d1=f*w';
return


function [d2]=distanceD2(f,d1)
if(size(d1,2)>1)
    d2=sqrt(sum(f.^2,2)*ones(1,size(d1,2))-d1.^2);
else
    d2=sqrt(sum(f.^2,2)*ones(1,size(d1,1))-d1.^2);
end
return


% Initializing a population of solutions
function [x,fitness]=initialize_pop(prob,def)
lb=prob.range(:,1)';ub=prob.range(:,2)';
x = bsxfun(@plus,lb,bsxfun(@times,lhsdesign(def.popSize,prob.nx),(ub-lb)));
[fitness]=Evaluate(x,prob,def);
return

%Evaluate solution/solutions
function [objVal]=Evaluate(x,prob,def)
func=str2func(def.problem_name);
% objVal=zeros(size(x,1),prob.nf+prob.ng+1);
[f,c,~] = func(prob.nf,x);
c = max(c,0);
if ~isempty(c)
    objVal = [f c sum(c,2)];
else
    objVal = [f c sum(c)*ones(size(x,1),1)];
end
return

% Generating the direction vectors
%Function to generate uniformly distributed weight vectors
function [N,W] = direction_vector(p1,p2,M)
[N,W] = gen_weight(p1,M);
if p2 > 0
    [N2,W2] = gen_weight(p2,M);
    N = N+N2;
    W = [W;W2*0.5+(1 - 0.5)/(M)];
end
return

function [N,W] = gen_weight(H,M)
N = nchoosek(H+M-1,M-1);
Temp = nchoosek(1:H+M-1,M-1)-repmat(0:M-2,nchoosek(H+M-1,M-1),1)-1;
W = zeros(N,M);
W(:,1) = Temp(:,1)-0;
for i = 2 : M-1
    W(:,i) = Temp(:,i)-Temp(:,i-1);
end
W(:,end) = H-Temp(:,end);
W = W/H;
return

% Function to write the all.dat file
function write_progress(x, fitness,prob,def, funCount, gen, p_name)
global alldata
flag=0;
if(funCount>=def.maxFunCount || def.statistics_all==1)
    flag=1;
end

if(flag==1)
    if(def.statistics_all)
        n=(gen-1)*def.popSize;
    else
        n=0;
    end
    for i=1:size(fitness,1)
        if(isempty(fitness(:,prob.nf+prob.ng+1))==1)
            alldata(n+i,:)=[gen i x(i,:) fitness(i,prob.nf+1:prob.nf+prob.ng) fitness(i,prob.nf+prob.ng+1) fitness(i,1:prob.nf)];
        else
            alldata(n+i,:)=[gen i x(i,:) fitness(i,prob.nf+1:prob.nf+prob.ng) fitness(i,prob.nf+prob.ng+1) fitness(i,1:prob.nf)];
        end
    end
    if(funCount>=def.maxFunCount)
        save(strcat(p_name,'_',num2str(def.nf),'_DBEAp_all','.mat'),'alldata');
    end
end
return

% Function to show the solutions over generations
function show_progress(fitness,prob,def,idealpoint,nadirpoint,w,fig_id,gen,flag,funCount)
if(def.display==1)
    figure(fig_id);
    m=prob.nf;
    N=size(fitness,1);
    feasible=[];
    infeasible=[];
    f=fitness(:,1:prob.nf);
    feas_id= fitness(:,prob.nf+prob.ng+1)==0;
    infeas_id=fitness(:,prob.nf+prob.ng+1)>0;
    F=(f-ones(N,1)*idealpoint)./(ones(N,1)*nadirpoint-ones(N,1)*idealpoint);

    if(~isempty(feas_id))
        if(flag==1)
            feasible=f(feas_id,:);
        else
            feasible=F(feas_id,:);
        end
    end
    if(~isempty(infeas_id))
        if(flag==1)
            infeasible=f(infeas_id,:);
        else
            infeasible=F(infeas_id,:);
        end
    end
    if(flag==2)
        for i=1:size(f,1)
            dn=sqrt(sum(w(i,:).^2));
            w(i,:)=w(i,:)/dn;
            if(m==2)
                O= [0 0];
            elseif(m==3)
                O=[0 0 0];
            end
            P=w(i,:);
            d1=F(i,1:prob.nf)*w(i,:)';
            cord=d1*w(i,:);
            if(m==2)
                plot([F(i,1) cord(1,1)],[F(i,2) cord(1,2)],'color','m','Marker','.','LineStyle','-','linewidth',2);hold on;
                plot([O(1,1) P(1,1)],[O(1,2) P(1,2)],'color','k','Marker','.','LineStyle','-','linewidth',2);
            elseif(m==3)
                plot3([F(i,1) cord(1,1)],[F(i,2) cord(1,2)],[F(i,3) cord(1,3)],'color','m','Marker','.','LineStyle','-','linewidth',2);hold on;
                plot3([O(1,1) cord(1,1)],[O(1,2) cord(1,2)],[O(1,3) cord(1,3)],'color','k','Marker','.','LineStyle','-','linewidth',2);
                view(111,28);
            end
            
        end
        hold on;
    end
    if(~isempty(feasible) && ~isempty(infeasible))
        if(m==2)
            plot(infeasible(:,1), infeasible(:,2), 'r*', ...
                feasible(:,1), feasible(:,2), 'b*');
            grid on;
        elseif(m==3)
            plot3(infeasible(:,1), infeasible(:,2), infeasible(:,3),'r*', ...
                feasible(:,1), feasible(:,2), feasible(:,3),'b*');
             view(109,32);
            grid on;
        end
    elseif(~isempty(infeasible))
        if(m==2)
            plot(infeasible(:,1), infeasible(:,2), 'r*');
            grid on
        elseif(m==3)
            plot3(infeasible(:,1), infeasible(:,2),infeasible(:,3), 'r*');
             view(109,32);
            grid on;
        end
    elseif(~isempty(feasible))
        if(m==2)
            plot(feasible(:,1), feasible(:,2), 'bo','linewidth',2);
            grid on;
        elseif(m==3)
            plot3(feasible(:,1), feasible(:,2),feasible(:,3), 'bo','markerfacecolor','blue', 'markersize',8);
            view(109,32);
            grid on;
        end
    end
    title(funCount);
    hold off;
    if(m==2)
        xlabel('f_1','FontSize',16,'FontName','Arial');
        ylabel('f_2','FontSize',16,'FontName','Arial');
    elseif(m==3)
        xlabel('f_1','FontSize',16,'FontName','Arial');
        ylabel('f_2','FontSize',16,'FontName','Arial');
        zlabel('f_3','FontSize',16,'FontName','Arial');
        view(109,32);
    end
end
return

%% nd_sort() - Find non-dominated points from a given set
% dir = 1, minimize all values
% dir = 2, maximimze all values
function [ndset,idx] = nd_rank_one(set1,dir)
if nargin == 1
	dir = 1;
end
[N,M] = size(set1);
switch(dir)
	case 1
		dom = nd_sort_min(set1, M, N);
	case 2
		dom = nd_sort_max(set1, M, N);
	otherwise
		error('wrong value of dir');
end

idx = [];
for i = 1:N
	if dom(i) == 0, idx = [idx i]; end
end
ndset = set1(idx(:),:);
return


%% nd_sort() for minimization
function [dom] = nd_sort_min(set1, M, N)
dom = zeros(1, N);
for i = 1:N
	f = repmat(set1(i,:), N, 1);
	dom_less = sum(f <= set1, 2);
	dom_more = sum(f >= set1, 2);
	for j = 1:N
		if dom_less(j) == M && dom_more(j) < M
			dom(j) = dom(j) + 1;
		end
	end
end
return


%% nd_sort() for maximization
function [dom] = nd_sort_max(set1, M, N)
dom = zeros(1, N);
for i = 1:N
	f = repmat(set1(i,:), N, 1);
	dom_less = sum(f <= set1, 2);
	dom_more = sum(f >= set1, 2);
	for j = 1:N
		if dom_more(j) == M && dom_less(j) < M
			dom(j) = dom(j) + 1;
		end
	end
end

function [prob]=load_problem_definition(def)
funh=str2func(def.problem_name);
prob=funh(def.nf);
return

function [newpoint]= DE_operator(x,p1,p2,p3,CR,F,domain)

parDim=size(x,2);

jrandom = ceil(rand*parDim);
oldpoint=x(p1,:);
randomarray = rand(parDim, 1);
deselect = randomarray<CR;
deselect(jrandom)=true;
newpoint = x(p1,:)+F*(x(p2,:)-x(p3,:));
newpoint(~deselect)=oldpoint(~deselect);

%repair the new value.
newpoint=max(newpoint, domain(:,1)');
newpoint=min(newpoint, domain(:,2)');

return


function [x1,x2]=SBX_crossover(x1,x2, prob,def)

if(rand(1)<=def.crossover_prob)
for i = 1:length(x1)
    if rand(1) <= 0.5
        [y1, y2] = op_SBX(x1(i), x2(i), prob.range(i,1), prob.range(i,2), def.crossover_sbx_eta);
        x1(i) = y1;
        x2(i) = y2;
    end
end
end

return

function [y1, y2] = op_SBX(x1, x2, x_min, x_max, eta)
% Make sure the variables are not the same
if abs(x1-x2) > 1.e-14
    % make sure x1 < x2
    if x2 < x1, tmp = x1; x1 = x2; x2 = tmp; end
    
    r = rand(1);
    beta = 1 + (2 * (x1-x_min) / (x2-x1));
    alpha = 2 - beta^-(eta+1);
    if r <= 1/alpha
        betaq = (r*alpha)^(1/(eta+1));
    else
        betaq = (1/(2-r*alpha))^(1/(eta+1));
    end
    y1 = 0.5 * (x1+x2 - betaq*(x2-x1));
    
    beta = 1 + (2 * (x_max-x2) / (x2-x1));
    alpha = 2 - beta^-(eta+1);
    if r <= 1/alpha
        betaq = (r*alpha)^(1/(eta+1));
    else
        betaq = (1/(2-r*alpha))^(1/(eta+1));
    end
    y2 = 0.5 * (x1+x2 + betaq*(x2-x1));
    
    if y1 < x_min, y1 = x_min; end
    if y1 > x_max, y1 = x_max; end
    
    if y2 < x_min, y2 = x_min; end
    if y2 > x_max, y2 = x_max; end
    
    if rand(1) <= 0.5, tmp = y1; y1 = y2; y2 = tmp; end
else
    y1 = x1;
    y2 = x2;
end
return

% Based on Roulette wheel selection
% input is an array of any real number values
% output is the selection of one of the elements marked by its index
% location

function [index] =  roulettewheelselection(arrayInput)

len = length(arrayInput);

% if input is one element then just return rightaway
if len ==1
    index =1;
    return;
end

if (~isempty(find(arrayInput<1, 1)))
    if (min(arrayInput) ~=0)
    arrayInput = 1/min(arrayInput)*arrayInput;
    else
    temp= arrayInput;
    temp(arrayInput==0) = inf;
    arrayInput = 1/min(temp)*arrayInput;
    end
end

temp = 0;
tempProb = zeros(1,len);

for i= 1:len
    tempProb(i) = temp + arrayInput(i);
    temp = tempProb(i);
end

i = fix(rand*floor(tempProb(end)))+1;
index = find(tempProb >= i, 1 );
return

function [x]=polymutation(x,prob,def)
def.mutation_prob=1/length(x);
for i = 1:length(x)
	if rand(1) <= def.mutation_prob
		x(i) = op_POLY(x(i),prob.range(i,1),prob.range(i,2),def.mutation_poly_eta);
	end
end

return

function x = op_POLY(x, x_min, x_max, eta_mu)
% Make sure the range is non-zero
if x_max-x_min > 0
	delta1 = (x-x_min) / (x_max-x_min);
	delta2 = (x_max-x) / (x_max-x_min);
	mut_pow = 1/(eta_mu+1);
	r = rand(1);
	if r <= 0.5
		xy = 1 - delta1;
		val = 2*r + (1-2*r) * xy^(eta_mu+1);
		deltaq = val^mut_pow - 1;
	else
		xy = 1 - delta2;
		val = 2*(1-r) + 2*(r-0.5) * xy^(eta_mu+1);
		deltaq = 1 - val^mut_pow;
	end
	x = x + deltaq*(x_max-x_min);
	if x < x_min, x = x_min; end
	if x > x_max, x = x_max; end
end
return